@@ -22,7 +22,7 @@ package MongoDB;
 # ABSTRACT: Official MongoDB Driver for Perl
 
 use version;
-our $VERSION = 'v0.999.998.1'; # TRIAL
+our $VERSION = 'v0.999.998.2'; # TRIAL
 
 # regexp_pattern was unavailable before 5.10, had to be exported to load the
 # function implementation on 5.10, and was automatically available in 5.10.1
@@ -51,6 +51,28 @@ if ( $] eq '5.010' ) {
     re->import('regexp_pattern');
 }
 
+#pod =method connect
+#pod
+#pod     $client = MongoDB->connect(); # localhost, port 27107
+#pod     $client = MongoDB->connect($host_uri);
+#pod     $client = MongoDB->connect($host_uri, $options);
+#pod
+#pod This function returns a L<MongoDB::MongoClient> object.  The first parameter is
+#pod used as the C<host> argument and must be a host name or L<connection string
+#pod URI|MongoDB::MongoClient/CONNECTION STRING URI>.  The second argument is
+#pod optional.  If provided, it must be a hash reference of constructor arguments
+#pod for L<MongoDB::MongoClient::new|MongoDB::MongoClient/ATTRIBUTES>.
+#pod
+#pod =cut
+
+sub connect {
+    my ($class, $host, $options) = @_;
+    $host ||= "mongodb://localhost";
+    $options ||= {};
+    $options->{host} = $host;
+    return MongoDB::MongoClient->new( $options );
+}
+
 sub force_double {
     if ( ref $_[0] ) {
         Carp::croak("Can't force a reference into a double");
@@ -79,9 +101,9 @@ MongoDB - Official MongoDB Driver for Perl
 
 =head1 VERSION
 
-version v0.999.998.1
+version v0.999.998.2
 
-This is the Alpha 1 release for v1.0.0.0.
+This is the Alpha 2 release for v1.0.0.0.
 
 =head1 ALPHA RELEASE NOTICE AND ROADMAP
 
@@ -93,24 +115,20 @@ important and will be delivered to a great extent, it will not be guaranteed.
 Using the v0.999.998.x series means that you understand that your code may break
 due to changes in the driver between now and the v1.0.0.0 stable release.
 
-This alpha 1 release includes these major changes:
+This Alpha 2 release includes these major changes:
 
 =over
 
 =item *
 
-All networking code is implemented in pure-Perl.  SSL support is provided by
-L<IO::Socket::SSL> (if installed).  Likewise, SASL authentication support is
-provided by L<Authen::SASL> backends (if installed).  This should improve
-portability and ease installation.
+The internal architecture for database write operations has been completely
+re-written.  For MongoDB 2.6 or later, writes are implemented using
+database write commands.  For earlier versions of MongoDB, writes continue
+to use legacy wire-protocol operations.
 
 =item *
 
-Server monitoring and failover are significantly improved.
-
-=item *
-
-Expanded use of exceptions for error handling.
+Compatibility with MongoDB 3.0.
 
 =back
 
@@ -119,7 +137,7 @@ L<MongoDB::Upgrading>.
 
 =head2 Roadmap
 
-Subsequent alphas will be released approximately monthly.  The v1.0.0.0 release
+Subsequent alphas will be released periodically.  The v1.0.0.0 release
 is expected in the middle of 2015.
 
 Some expected (but not guaranteed) changes in future releases include:
@@ -128,15 +146,7 @@ Some expected (but not guaranteed) changes in future releases include:
 
 =item *
 
-The driver will become pure-Perl capable, using the Moo framework instead of Moose.
-
-=item *
-
-BSON encoding will be extracted to a separate module, with both pure-Perl and C variants available.
-
-=item *
-
-Transformation of Perl data structures to/from BSON will become more customizable.
+A revised CRUD API will be added, which will be consistent across all officially-maintained "next-generation" language drivers.  Many legacy CRUD method will be deprecated.
 
 =item *
 
@@ -152,28 +162,44 @@ Some configuration options and method return values will be implemented with obj
 
 =item *
 
-Various internal changes to support new protocol capabilities of MongoDB 2.6 and later.
-
-=item *
-
-The driver will have a smaller total dependency tree.
-
-=item *
-
 Documentation will be significantly revised.
 
 =back
 
+In order to avoid holding up v1.0.0.0, the following changes will be deferred
+until after the v1.0.0.0 release:
+
+=over
+
+=item *
+
+The driver will become pure-Perl capable, using the Moo framework instead of Moose.  This will significantly reduce the size of the total dependency tree.
+
+=item *
+
+BSON encoding will be extracted to a separate module, with both pure-Perl and C variants available.
+
+=item *
+
+Transformation of Perl data structures to/from BSON will become more customizable.
+
+=back
+
 =head1 SYNOPSIS
 
     use MongoDB;
 
-    my $client     = MongoDB::MongoClient->new(host => 'localhost', port => 27017);
-    my $database   = $client->get_database( 'foo' );
-    my $collection = $database->get_collection( 'bar' );
+    # short-hand
+    my $client     = MongoDB->connect('mongodb://localhost');
+    my $collection = $client->ns('foo.bar'); # database foo, collection bar
     my $id         = $collection->insert({ some => 'data' });
     my $data       = $collection->find_one({ _id => $id });
 
+    # long-hand
+    my $client     = MongoDB::MongoClient->new(host => 'mongodb://localhost');
+    my $database   = $client->get_database( 'foo' );
+    my $collection = $database->get_collection( 'bar' );
+
 =head1 DESCRIPTION
 
 This is the official Perl driver for MongoDB.  MongoDB is an open-source
@@ -239,6 +265,20 @@ classes may return other objects for specific features or functions.
 See the documentation of those classes for more details or the
 L<MongoDB Perl Driver Tutorial|MongoDB::Tutorial> for an example.
 
+=head1 METHODS
+
+=head2 connect
+
+    $client = MongoDB->connect(); # localhost, port 27107
+    $client = MongoDB->connect($host_uri);
+    $client = MongoDB->connect($host_uri, $options);
+
+This function returns a L<MongoDB::MongoClient> object.  The first parameter is
+used as the C<host> argument and must be a host name or L<connection string
+URI|MongoDB::MongoClient/CONNECTION STRING URI>.  The second argument is
+optional.  If provided, it must be a hash reference of constructor arguments
+for L<MongoDB::MongoClient::new|MongoDB::MongoClient/ATTRIBUTES>.
+
 =head1 SEMANTIC VERSIONING SCHEME
 
 Starting with MongoDB v0.704.0.0, the driver will be using a modified
@@ -318,7 +358,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Andrew Page Andrey Khozov Ashley Willis Ask Bjørn Hansen Brendan W. McAdams Casey Rojas Christian Sturm Colin Cyr danny David Morrison Nadle Steinbrunner Storch D. Ilmari Mannsåker Eric Daniels Gerard Goossen Graham Barr Jason Carey Toffaletti Johann Rolschewski Joseph Harnish Joshua Juran J. Stewart Kamil Slowikowski Ken Williams mapbuh Matthew Shopsin Michael Langner Rotmanov Mike Dirolf Friedman nightlord nightsailer Nuno Carvalho Orlando Vazquez Othello Maurer Robin Lee Roman Yerin Ronald J Kimball Stephen Oberholtzer Steve Sanbeg Stuart Watt Uwe Voelker Whitney.Jackson
+=for stopwords Andrew Page Andrey Khozov Ashley Willis Ask Bjørn Hansen Brendan W. McAdams Casey Rojas Christian Sturm Colin Cyr danny David Morrison Nadle Steinbrunner Storch D. Ilmari Mannsåker Eric Daniels Gerard Goossen Graham Barr Jason Carey Toffaletti Johann Rolschewski Joseph Harnish Joshua Juran J. Stewart Kamil Slowikowski Ken Williams mapbuh Matthew Shopsin Michael Langner Rotmanov Mike Dirolf nightlord nightsailer Nuno Carvalho Orlando Vazquez Othello Maurer Robin Lee Roman Yerin Ronald J Kimball Stephen Oberholtzer Steve Sanbeg Stuart Watt Uwe Voelker Whitney.Jackson
 
 =over 4
 
@@ -444,10 +484,6 @@ Mike Dirolf <mike@mongodb.com>
 
 =item *
 
-Mike Friedman <mike.friedman@mongodb.com>
-
-=item *
-
 nightlord <zzh_621@yahoo.com>
 
 =item *
@@ -502,7 +538,7 @@ Whitney.Jackson <whjackson@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by MongoDB, Inc..
+This software is Copyright (c) 2015 by MongoDB, Inc..
 
 This is free software, licensed under:
 
