@@ -19,22 +19,112 @@ package MongoDB::BulkWriteView;
 # ABSTRACT: Bulk write operations against a query document
 
 use version;
-our $VERSION = 'v0.999.998.1'; # TRIAL
+our $VERSION = 'v0.999.998.2'; # TRIAL
 
 use Moose;
+
+use Syntax::Keyword::Junction qw/any/;
+use boolean;
 use namespace::clean -except => 'meta';
 
-# done in two parts to work around a moose bug: https://github.com/moose/Moose/pull/19
-with qw(
-  MongoDB::Role::_View
-  MongoDB::Role::_Writeable
+# A hash reference containing a MongoDB query document
+has _query => (
+    is       => 'ro',
+    isa      => 'IxHash',
+    coerce   => 1,
+    required => 1
 );
 
-with qw(
-  MongoDB::Role::_Updater
-  MongoDB::Role::_Remover
+# Originating bulk write object for executing write operations.
+has _bulk => (
+    is       => 'ro',
+    isa      => 'MongoDB::BulkWrite',
+    required => 1,
+    handles  => [qw/_enqueue_write/]
 );
 
+has _upsert => (
+    is      => 'ro',
+    isa     => 'boolean',
+    default => sub { false },
+);
+
+sub upsert {
+    my ($self) = @_;
+    unless ( @_ == 1 ) {
+        confess "the upsert method takes no arguments";
+    }
+    return $self->new( %$self, _upsert => true );
+}
+
+sub update {
+    push @_, "update";
+    goto &_update;
+}
+
+sub update_one {
+    push @_, "update_one";
+    goto &_update;
+}
+
+sub replace_one {
+    push @_, "replace_one";
+    goto &_update;
+}
+
+sub _update {
+    my $method = pop @_;
+    my ( $self, $doc ) = @_;
+
+    unless ( @_ == 2 && ref $doc eq any(qw/HASH ARRAY Tie::IxHash/) ) {
+        confess "argument to $method must be a single hashref, arrayref or Tie::IxHash";
+    }
+
+    if ( ref $doc eq 'ARRAY' ) {
+        confess "array reference to $method must have key/value pairs"
+          if @$doc % 2;
+        $doc = Tie::IxHash->new(@$doc);
+    }
+    elsif ( ref $doc eq 'HASH' ) {
+        $doc = Tie::IxHash->new(%$doc);
+    }
+
+    my @keys = $doc->Keys;
+    if ( $method eq 'replace_one' ) {
+        if ( my @bad = grep { substr( $_, 0, 1 ) eq '$' } @keys ) {
+            confess "$method document can't have '\$' prefixed field names: @bad";
+        }
+    }
+    else {
+        if ( my @bad = grep { substr( $_, 0, 1 ) ne '$' } @keys ) {
+            confess "$method document can't have non- '\$' prefixed field names: @bad";
+        }
+    }
+
+    my $update = {
+        q      => $self->_query,
+        u      => $doc,
+        multi  => $method eq 'update' ? true : false,
+        upsert => boolean( $self->_upsert ),
+    };
+
+    $self->_enqueue_write( [ update => $update ] );
+
+    return;
+}
+
+sub remove {
+    my ($self) = @_;
+    $self->_enqueue_write( [ delete => { q => $self->_query, limit => 0 } ] );
+    return;
+}
+
+sub remove_one {
+    my ($self) = @_;
+    $self->_enqueue_write( [ delete => { q => $self->_query, limit => 1 } ] );
+    return;
+}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -51,7 +141,7 @@ MongoDB::BulkWriteView - Bulk write operations against a query document
 
 =head1 VERSION
 
-version v0.999.998.1
+version v0.999.998.2
 
 =head1 SYNOPSIS
 
@@ -164,7 +254,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by MongoDB, Inc..
+This software is Copyright (c) 2015 by MongoDB, Inc..
 
 This is free software, licensed under:
 
