@@ -19,7 +19,7 @@ package MongoDB::CommandResult;
 # ABSTRACT: MongoDB generic command result document
 
 use version;
-our $VERSION = 'v0.999.998.1'; # TRIAL
+our $VERSION = 'v0.999.998.2'; # TRIAL
 
 use Moose;
 use MongoDB::Error;
@@ -51,6 +51,17 @@ has address => (
     required => 1,
 );
 
+#pod =method last_code
+#pod
+#pod Error code (if any) or 0 if there was no error.
+#pod
+#pod =cut
+
+sub last_code {
+    my ($self) = @_;
+    return $self->result->{code} || 0;
+}
+
 #pod =method last_errmsg
 #pod
 #pod Error string (if any) or the empty string if there was no error.
@@ -65,27 +76,29 @@ sub last_errmsg {
     return "";
 }
 
-sub assert {
+#pod =method last_wtimeout
+#pod
+#pod True if a write concern timed out or false otherwise.
+#pod
+#pod =cut
+
+sub last_wtimeout {
     my ($self) = @_;
-    if ( ! $self->result->{ok} ) {
-        my $err = $self->last_errmsg;
-        my $code = $self->result->{code};
-        my $error_class;
+    return !!$self->result->{wtimeout};
+}
 
-        # XXX should we be detecting write/writeConcern/etc errors here?
-        if ( $err =~ /^(?:not master|node is recovering)/ ) {
-            $error_class = "MongoDB::NotMasterError";
-        }
-        else {
-            $error_class = "MongoDB::DatabaseError";
-        }
+#pod =method assert
+#pod
+#pod Throws an exception if the command failed.
+#pod
+#pod =cut
+
+sub assert {
+    my ($self, $default_class) = @_;
+
+    $self->_throw_database_error( $default_class )
+        if ! $self->result->{ok};
 
-        $error_class->throw(
-            result => $self,
-            ( length($err)   ? ( message => $err )  : () ),
-            ( defined($code) ? ( code    => $code ) : () ),
-        );
-    }
     return 1;
 }
 
@@ -105,7 +118,7 @@ MongoDB::CommandResult - MongoDB generic command result document
 
 =head1 VERSION
 
-version v0.999.998.1
+version v0.999.998.2
 
 =head1 DESCRIPTION
 
@@ -124,10 +137,22 @@ Address ("host:port") of server that ran the command
 
 =head1 METHODS
 
+=head2 last_code
+
+Error code (if any) or 0 if there was no error.
+
 =head2 last_errmsg
 
 Error string (if any) or the empty string if there was no error.
 
+=head2 last_wtimeout
+
+True if a write concern timed out or false otherwise.
+
+=head2 assert
+
+Throws an exception if the command failed.
+
 =head1 AUTHORS
 
 =over 4
@@ -152,7 +177,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by MongoDB, Inc..
+This software is Copyright (c) 2015 by MongoDB, Inc..
 
 This is free software, licensed under:
 
