@@ -20,7 +20,7 @@ package MongoDB::Cursor;
 # ABSTRACT: A lazy cursor for Mongo query results
 
 use version;
-our $VERSION = 'v0.999.998.1'; # TRIAL
+our $VERSION = 'v0.999.998.2'; # TRIAL
 
 use Moose;
 use MongoDB;
@@ -104,75 +104,13 @@ $MongoDB::Cursor::timeout = 30000;
 
 with 'MongoDB::Role::_Cursor';
 
-# general attributes
-
-has _client => (
-    is => 'rw',
-    isa => 'MongoDB::MongoClient',
-    required => 1,
-);
-
-has _ns => (
-    is => 'ro',
-    isa => 'Str',
-    required => 1,
-);
-
 # attributes for sending a query
-
-has _query => (
-    is => 'rw',
-    isa => 'MongoDBQuery',
-    required => 1,
-    coerce => 1,
-    writer => '_set_query',
-);
-
-has _fields => (
-    is => 'rw',
-    required => 0,
-);
-
-has _limit => (
-    is => 'rw',
-    isa => 'Int',
-    required => 0,
-    default => 0,
-);
-
-has _batch_size => (
-    is => 'rw',
-    isa => 'Int',
-    required => 0,
-    default => 0,
-);
-
-has _skip => (
-    is => 'rw',
-    isa => 'Int',
-    required => 0,
-    default => 0,
-);
-
-has _query_options => (
+has query => (
     is => 'ro',
-    isa => 'HashRef',
-    default => sub { { slave_ok => !! $MongoDB::Cursor::slave_okay } },
+    isa => 'MongoDB::_Query',
+    required => 1,
 );
 
-has _read_preference => (
-    is      => 'ro',
-    isa     => 'ReadPreference',
-    lazy    => 1,
-    builder => '_build__read_preference',
-    writer  => '_set_read_preference',
-);
-
-sub _build__read_preference {
-    my ($self) = @_;
-    return $self->_client->read_preference;
-}
-
 # lazy result attribute
 has result => (
     is        => 'ro',
@@ -186,16 +124,7 @@ has result => (
 # this does the query if it hasn't been done yet
 sub _build_result {
     my ($self) = @_;
-    return $self->_client->send_query(
-        $self->_ns,
-        $self->_query,
-        $self->_fields,
-        $self->_skip,
-        $self->_limit,
-        $self->_batch_size,
-        $self->_query_options,
-        $self->_read_preference,
-    );
+    $self->query->execute;
 }
 
 #--------------------------------------------------------------------------#
@@ -227,6 +156,8 @@ sub _build_result {
 #pod
 #pod See L<MongoDB::MongoClient/query_timeout>.
 #pod
+#pod Returns this cursor for chaining operations.
+#pod
 #pod =cut
 
 sub immortal {
@@ -234,20 +165,28 @@ sub immortal {
     confess "cannot set immortal after querying"
         if $self->started_iterating;
 
-    $self->_query_options->{immortal} = !!$bool;
+    $self->query->noCursorTimeout(!!$bool);
     return $self;
 }
 
-#pod =head2 fields (\%f)
+#pod =head2 fields
 #pod
 #pod     $coll->insert({name => "Fred", age => 20});
-#pod     my $cursor = $coll->query->fields({ name => 1 });
+#pod     my $cursor = $coll->find->fields({ name => 1 });
 #pod     my $obj = $cursor->next;
 #pod     $obj->{name}; "Fred"
 #pod     $obj->{age}; # undef
 #pod
-#pod Selects which fields are returned.
-#pod The default is all fields.  _id is always returned.
+#pod Selects which fields are returned.  The default is all fields.  When fields
+#pod are specified, _id is returned by default, but this can be disabled by
+#pod explicitly setting it to "0".  E.g.  C<< _id => 0 >>. Argument is either a
+#pod hash reference, a L<Tie::IxHash> or an array reference of key/value pairs.
+#pod
+#pod See L<Limit fields to
+#pod return|http://docs.mongodb.org/manual/tutorial/project-fields-from-query-results/>
+#pod in the MongoDB documentation for details.
+#pod
+#pod Returns this cursor for chaining operations.
 #pod
 #pod =cut
 
@@ -258,63 +197,64 @@ sub fields {
     confess 'not a hash reference'
 	    unless ref $f eq 'HASH' || ref $f eq 'Tie::IxHash';
 
-    $self->_fields($f);
+    $self->query->projection($f);
     return $self;
 }
 
 #pod =head2 sort ($order)
 #pod
 #pod     # sort by name, descending
-#pod     my $sort = {"name" => -1};
-#pod     $cursor = $coll->query->sort($sort);
+#pod     $cursor->sort([name => -1]);
+#pod
+#pod Adds a sort to the query.  Argument is either a hash reference or a
+#pod L<Tie::IxHash> or an array reference of key/value pairs.  Because hash
+#pod references are not ordered, do not use them for more than one key.
 #pod
-#pod Adds a sort to the query.  Argument is either
-#pod a hash reference or a Tie::IxHash.
 #pod Returns this cursor for chaining operations.
 #pod
 #pod =cut
 
 sub sort {
-    my ($self, $order) = @_;
+    my ( $self, $order ) = @_;
     confess "cannot set sort after querying"
-	if $self->started_iterating;
-    confess 'not a hash reference'
-	    unless ref $order eq 'HASH' || ref $order eq 'Tie::IxHash';
+      if $self->started_iterating;
 
-    $self->_query->set_modifier('$orderby', $order);
+    $self->query->sort($order);
     return $self;
 }
 
 
-#pod =head2 limit ($num)
+#pod =head2 limit
 #pod
-#pod     $per_page = 20;
-#pod     $cursor = $coll->query->limit($per_page);
+#pod     $cursor->limit(20);
+#pod
+#pod Sets cursor to return a maximum of N results.
 #pod
-#pod Returns a maximum of N results.
 #pod Returns this cursor for chaining operations.
 #pod
 #pod =cut
 
 sub limit {
-    my ($self, $num) = @_;
+    my ( $self, $num ) = @_;
     confess "cannot set limit after querying"
-	if $self->started_iterating;
-    $self->_limit($num);
+      if $self->started_iterating;
+    $self->query->limit($num);
     return $self;
 }
 
 
-#pod =head2 max_time_ms( $millis )
+#pod =head2 max_time_ms
 #pod
-#pod     $cursor = $coll->query->max_time_ms( 500 );
+#pod     $cursor->max_time_ms( 500 );
 #pod
-#pod Causes the server to abort the operation if the specified time in 
-#pod milliseconds is exceeded. 
+#pod Causes the server to abort the operation if the specified time in milliseconds
+#pod is exceeded.
+#pod
+#pod Returns this cursor for chaining operations.
 #pod
 #pod =cut
 
-sub max_time_ms { 
+sub max_time_ms {
     my ( $self, $num ) = @_;
     $num = 0 unless defined $num;
     confess "max_time_ms must be non-negative"
@@ -322,14 +262,14 @@ sub max_time_ms {
     confess "can not set max_time_ms after querying"
       if $self->started_iterating;
 
-    $self->_query->set_modifier( '$maxTimeMS', $num );
+    $self->query->maxTimeMS( $num );
     return $self;
 
 }
 
-#pod =head2 tailable ($bool)
+#pod =head2 tailable
 #pod
-#pod     $cursor->query->tailable(1);
+#pod     $cursor->tailable(1);
 #pod
 #pod If a cursor should be tailable.  Tailable cursors can only be used on capped
 #pod collections and are similar to the C<tail -f> command: they never die and keep
@@ -339,6 +279,10 @@ sub max_time_ms {
 #pod
 #pod Boolean value, defaults to 0.
 #pod
+#pod If you want the tailable cursor to block for a few seconds, use
+#pod L</tailable_await> instead.  B<Note> calling this with a false value
+#pod disables tailing, even if C<tailable_await> was previously called.
+#pod
 #pod Returns this cursor for chaining operations.
 #pod
 #pod =cut
@@ -348,83 +292,107 @@ sub tailable {
     confess "cannot set tailable after querying"
         if $self->started_iterating;
 
-    $self->_query_options->{tailable} = !!$bool;
+    $self->query->cursorType($bool ? 'tailable' : 'non_tailable');
     return $self;
 }
 
+#pod =head2 tailable_await
+#pod
+#pod     $cursor->tailable_await(1);
+#pod
+#pod Sets a cursor to be tailable and block for a few seconds if no data
+#pod is immediately available.
+#pod
+#pod Boolean value, defaults to 0.
+#pod
+#pod If you want the tailable cursor without blocking, use L</tailable> instead.
+#pod B<Note> calling this with a false value disables tailing, even if C<tailable>
+#pod was previously called.
+#pod
+#pod =cut
 
+sub tailable_await {
+    my ( $self, $bool ) = @_;
+    confess "cannot set tailable_await after querying"
+        if $self->started_iterating;
 
-#pod =head2 skip ($num)
+    $self->query->cursorType($bool ? 'tailable_await' : 'non_tailable');
+    return $self;
+}
+
+#pod =head2 skip
 #pod
-#pod     $page_num = 7;
-#pod     $per_page = 100;
-#pod     $cursor = $coll->query->limit($per_page)->skip($page_num * $per_page);
+#pod     $cursor->skip( 50 );
 #pod
-#pod Skips the first N results. Returns this cursor for chaining operations.
+#pod Skips the first N results.
 #pod
-#pod See also core documentation on limit: L<http://dochub.mongodb.org/core/limit>.
+#pod Returns this cursor for chaining operations.
 #pod
 #pod =cut
 
 sub skip {
-    my ($self, $num) = @_;
+    my ( $self, $num ) = @_;
+    confess "skip must be non-negative"
+      if $num < 0;
     confess "cannot set skip after querying"
-	if $self->started_iterating;
+      if $self->started_iterating;
 
-    $self->_skip($num);
+    $self->query->skip($num);
     return $self;
 }
 
 #pod =head2 snapshot
 #pod
-#pod     my $cursor = $coll->query->snapshot;
+#pod     $cursor->snapshot;
 #pod
-#pod Uses snapshot mode for the query.  Snapshot mode assures no
-#pod duplicates are returned, or objects missed, which were present
-#pod at both the start and end of the query's execution (if an object
-#pod is new during the query, or deleted during the query, it may or
-#pod may not be returned, even with snapshot mode).  Note that short
-#pod query responses (less than 1MB) are always effectively
-#pod snapshotted.  Currently, snapshot mode may not be used with
-#pod sorting or explicit hints.
+#pod Uses snapshot mode for the query.  Snapshot mode assures no duplicates are
+#pod returned due an intervening write relocating a document.  Note that if an
+#pod object is inserted, updated or deleted during the query, it may or may not
+#pod be returned when snapshot mode is enabled. Short query responses (less than
+#pod 1MB) are always effectively snapshotted.  Currently, snapshot mode may not
+#pod be used with sorting or explicit hints.
+#pod
+#pod Returns this cursor for chaining operations.
 #pod
 #pod =cut
 
+# XXX this doesn't allow turning it off!  That's inconsistent
+# with the rest of the cursor API
 sub snapshot {
     my ($self) = @_;
     confess "cannot set snapshot after querying"
-	if $self->started_iterating;
+      if $self->started_iterating;
 
-    $self->_query->set_modifier('$snapshot', 1);
+    $self->query->modifiers->{'$snapshot'} = 1;
     return $self;
 }
 
 #pod =head2 hint
 #pod
-#pod     my $cursor = $coll->query->hint({'x' => 1});
-#pod     my $cursor = $coll->query->hint(['x', 1]);
-#pod     my $cursor = $coll->query->hint('x_1');
+#pod     $cursor->hint({'x' => 1});
+#pod     $cursor->hint(['x', 1]);
+#pod     $cursor->hint('x_1');
 #pod
 #pod Force Mongo to use a specific index for a query.
 #pod
+#pod Returns this cursor for chaining operations.
+#pod
 #pod =cut
 
 sub hint {
-    my ($self, $index) = @_;
+    my ( $self, $index ) = @_;
     confess "cannot set hint after querying"
-        if $self->started_iterating;
+      if $self->started_iterating;
 
     # $index must either be a string or a reference to an array, hash, or IxHash
-    if (ref $index eq 'ARRAY') {
-
+    if ( ref $index eq 'ARRAY' ) {
         $index = Tie::IxHash->new(@$index);
-
-    } elsif (ref $index && !(ref $index eq 'HASH' || ref $index eq 'Tie::IxHash')) {
-
+    }
+    elsif ( ref $index && !( ref $index eq 'HASH' || ref $index eq 'Tie::IxHash' ) ) {
         confess 'not a hash reference';
     }
 
-    $self->_query->set_modifier('$hint', $index);
+    $self->query->modifiers->{'$hint'} = $index;
     return $self;
 }
 
@@ -437,11 +405,16 @@ sub hint {
 #pod
 #pod Boolean value, defaults to 0.
 #pod
+#pod Returns this cursor for chaining operations.
+#pod
 #pod =cut
 
 sub partial {
     my ($self, $value) = @_;
-    $self->_query_options->{partial} = !! $value;
+    confess "cannot set partial after querying"
+      if $self->started_iterating;
+
+    $self->query->allowPartialResults( !! $value );
 
     # XXX returning self is an API change but more consistent with other cursor methods
     return $self;
@@ -449,8 +422,8 @@ sub partial {
 
 #pod =head2 read_preference
 #pod
-#pod     my $cursor = $coll->find()->read_preference($read_preference_object);
-#pod     my $cursor = $coll->find()->read_preference('secondary', [{foo => 'bar'}]);
+#pod     $cursor->read_preference($read_preference_object);
+#pod     $cursor->read_preference('secondary', [{foo => 'bar'}]);
 #pod
 #pod Sets read preference for the cursor's connection.
 #pod
@@ -460,16 +433,18 @@ sub partial {
 #pod mode and tag set list as described in the L<MongoDB::ReadPreference>
 #pod documentation.
 #pod
-#pod Returns $self so that this method can be chained.
+#pod Returns this cursor for chaining operations.
 #pod
 #pod =cut
 
 sub read_preference {
     my $self = shift;
+    confess "cannot set read preference after querying"
+      if $self->started_iterating;
 
     my $type = ref $_[0];
     if ( $type eq 'MongoDB::ReadPreference' ) {
-        $self->_set_read_preference( $_[0] );
+        $self->query->read_preference( $_[0] );
     }
     else {
         my $mode     = shift || 'primary';
@@ -478,13 +453,13 @@ sub read_preference {
             mode => $mode,
             ( $tag_sets ? ( tag_sets => $tag_sets ) : () )
         );
-        $self->_set_read_preference($rp);
+        $self->query->read_preference($rp);
     }
 
     return $self;
 }
 
-#pod =head2 slave_okay
+#pod =head2 slave_okay (DEPRECATED)
 #pod
 #pod     $cursor->slave_okay(1);
 #pod
@@ -492,13 +467,25 @@ sub read_preference {
 #pod
 #pod Boolean value, defaults to 0.
 #pod
-#pod Returns the cursor object
+#pod Returns this cursor for chaining operations.
 #pod
 #pod =cut
 
 sub slave_okay {
     my ($self, $value) = @_;
-    $self->_query_options->{slave_ok} = !! $value;
+    confess "cannot set slave_ok after querying"
+      if $self->started_iterating;
+
+    if ( $value ) {
+        # if not 'primary', then slave_ok is already true, so leave alone
+        if ( $self->query->read_preference->mode eq 'primary' ) {
+            # secondaryPreferred is how mongos interpretes slave_ok
+            $self->query->read_preference('secondaryPreferred');
+        }
+    }
+    else {
+        $self->query->read_preference('primary');
+    }
 
     # XXX returning self is an API change but more consistent with other cursor methods
     return $self;
@@ -515,10 +502,7 @@ sub slave_okay {
 #pod
 #pod This will tell you the type of cursor used, the number of records the DB had to
 #pod examine as part of this query, the number of records returned by the query, and
-#pod the time in milliseconds the query took to execute.  Requires L<boolean> package.
-#pod
-#pod C<explain> resets the cursor, so calling C<next> or C<has_next> after an explain
-#pod will requery the database.
+#pod the time in milliseconds the query took to execute.
 #pod
 #pod See also core documentation on explain:
 #pod L<http://dochub.mongodb.org/core/explain>.
@@ -527,23 +511,16 @@ sub slave_okay {
 
 sub explain {
     my ($self) = @_;
-    my $temp = $self->_limit;
-    if ($self->_limit > 0) {
-        $self->_limit($self->_limit * -1);
-    }
 
-    my $old_query = $self->_query;
-    my $new_query = $old_query->clone;
-    $new_query->set_modifier('$explain', boolean::true);
+    my $new_query = $self->query->clone;
+    $new_query->modifiers->{'$explain'} = true;
 
-    $self->_set_query( $new_query  );
+    # per David Storch, drivers *must* send a negative limit to instruct
+    # the query planner analysis module to add a LIMIT stage.  For older
+    # explain implementations, it also ensures a cursor isn't left open.
+    $new_query->limit( -1 * abs( $new_query->limit ) );
 
-    my $retval = $self->reset->next;
-
-    $self->_set_query( $old_query );
-    $self->reset->limit($temp);
-
-    return $retval;
+    return $new_query->execute->next;
 }
 
 #pod =head2 count($all?)
@@ -558,25 +535,25 @@ sub explain {
 #pod =cut
 
 sub count {
-    my ($self, $all) = @_;
+    my ($self, $limit_skip) = @_;
     # XXX deprecate this unintuitive API?
 
-    my ($db, $coll) = $self->_ns =~ m/^([^\.]+)\.(.*)/;
-    my $cmd = new Tie::IxHash(count => $coll);
+    my $cmd = new Tie::IxHash(count => $self->query->coll_name);
 
-    $cmd->Push(query => $self->_query->query_doc);
+    $cmd->Push(query => $self->query->filter);
 
-    if ($all) {
-        $cmd->Push(limit => $self->_limit) if $self->_limit;
-        $cmd->Push(skip => $self->_skip) if $self->_skip;
+    if ($limit_skip) {
+        $cmd->Push(limit => $self->query->limit) if $self->query->limit;
+        $cmd->Push(skip => $self->query->skip) if $self->query->skip;
     }
 
-    if (my $hint = $self->_query->get_modifier('$hint')) {
+    if (my $hint = $self->query->modifiers->{'$hint'}) {
         $cmd->Push(hint => $hint);
     }
 
     my $result = try {
-        $self->_client->get_database($db)->_try_run_command($cmd);
+        my $db = $self->query->client->get_database( $self->query->db_name );
+        $db->run_command( $cmd, $self->query->read_preference );
     }
     catch {
         # if there was an error, check if it was the "ns missing" one that means the
@@ -695,6 +672,8 @@ __PACKAGE__->meta->make_immutable;
 
 1;
 
+# vim: ts=4 sts=4 sw=4 et tw=75:
+
 __END__
 
 =pod
@@ -707,7 +686,7 @@ MongoDB::Cursor - A lazy cursor for Mongo query results
 
 =head1 VERSION
 
-version v0.999.998.1
+version v0.999.998.2
 
 =head1 SYNOPSIS
 
@@ -783,45 +762,58 @@ your connection.
 
 See L<MongoDB::MongoClient/query_timeout>.
 
-=head2 fields (\%f)
+Returns this cursor for chaining operations.
+
+=head2 fields
 
     $coll->insert({name => "Fred", age => 20});
-    my $cursor = $coll->query->fields({ name => 1 });
+    my $cursor = $coll->find->fields({ name => 1 });
     my $obj = $cursor->next;
     $obj->{name}; "Fred"
     $obj->{age}; # undef
 
-Selects which fields are returned.
-The default is all fields.  _id is always returned.
+Selects which fields are returned.  The default is all fields.  When fields
+are specified, _id is returned by default, but this can be disabled by
+explicitly setting it to "0".  E.g.  C<< _id => 0 >>. Argument is either a
+hash reference, a L<Tie::IxHash> or an array reference of key/value pairs.
+
+See L<Limit fields to
+return|http://docs.mongodb.org/manual/tutorial/project-fields-from-query-results/>
+in the MongoDB documentation for details.
+
+Returns this cursor for chaining operations.
 
 =head2 sort ($order)
 
     # sort by name, descending
-    my $sort = {"name" => -1};
-    $cursor = $coll->query->sort($sort);
+    $cursor->sort([name => -1]);
+
+Adds a sort to the query.  Argument is either a hash reference or a
+L<Tie::IxHash> or an array reference of key/value pairs.  Because hash
+references are not ordered, do not use them for more than one key.
 
-Adds a sort to the query.  Argument is either
-a hash reference or a Tie::IxHash.
 Returns this cursor for chaining operations.
 
-=head2 limit ($num)
+=head2 limit
 
-    $per_page = 20;
-    $cursor = $coll->query->limit($per_page);
+    $cursor->limit(20);
+
+Sets cursor to return a maximum of N results.
 
-Returns a maximum of N results.
 Returns this cursor for chaining operations.
 
-=head2 max_time_ms( $millis )
+=head2 max_time_ms
 
-    $cursor = $coll->query->max_time_ms( 500 );
+    $cursor->max_time_ms( 500 );
 
-Causes the server to abort the operation if the specified time in 
-milliseconds is exceeded. 
+Causes the server to abort the operation if the specified time in milliseconds
+is exceeded.
 
-=head2 tailable ($bool)
+Returns this cursor for chaining operations.
 
-    $cursor->query->tailable(1);
+=head2 tailable
+
+    $cursor->tailable(1);
 
 If a cursor should be tailable.  Tailable cursors can only be used on capped
 collections and are similar to the C<tail -f> command: they never die and keep
@@ -831,39 +823,56 @@ They are often used for getting log messages.
 
 Boolean value, defaults to 0.
 
+If you want the tailable cursor to block for a few seconds, use
+L</tailable_await> instead.  B<Note> calling this with a false value
+disables tailing, even if C<tailable_await> was previously called.
+
 Returns this cursor for chaining operations.
 
-=head2 skip ($num)
+=head2 tailable_await
 
-    $page_num = 7;
-    $per_page = 100;
-    $cursor = $coll->query->limit($per_page)->skip($page_num * $per_page);
+    $cursor->tailable_await(1);
 
-Skips the first N results. Returns this cursor for chaining operations.
+Sets a cursor to be tailable and block for a few seconds if no data
+is immediately available.
 
-See also core documentation on limit: L<http://dochub.mongodb.org/core/limit>.
+Boolean value, defaults to 0.
+
+If you want the tailable cursor without blocking, use L</tailable> instead.
+B<Note> calling this with a false value disables tailing, even if C<tailable>
+was previously called.
+
+=head2 skip
+
+    $cursor->skip( 50 );
+
+Skips the first N results.
+
+Returns this cursor for chaining operations.
 
 =head2 snapshot
 
-    my $cursor = $coll->query->snapshot;
+    $cursor->snapshot;
 
-Uses snapshot mode for the query.  Snapshot mode assures no
-duplicates are returned, or objects missed, which were present
-at both the start and end of the query's execution (if an object
-is new during the query, or deleted during the query, it may or
-may not be returned, even with snapshot mode).  Note that short
-query responses (less than 1MB) are always effectively
-snapshotted.  Currently, snapshot mode may not be used with
-sorting or explicit hints.
+Uses snapshot mode for the query.  Snapshot mode assures no duplicates are
+returned due an intervening write relocating a document.  Note that if an
+object is inserted, updated or deleted during the query, it may or may not
+be returned when snapshot mode is enabled. Short query responses (less than
+1MB) are always effectively snapshotted.  Currently, snapshot mode may not
+be used with sorting or explicit hints.
+
+Returns this cursor for chaining operations.
 
 =head2 hint
 
-    my $cursor = $coll->query->hint({'x' => 1});
-    my $cursor = $coll->query->hint(['x', 1]);
-    my $cursor = $coll->query->hint('x_1');
+    $cursor->hint({'x' => 1});
+    $cursor->hint(['x', 1]);
+    $cursor->hint('x_1');
 
 Force Mongo to use a specific index for a query.
 
+Returns this cursor for chaining operations.
+
 =head2 partial
 
     $cursor->partial(1);
@@ -873,10 +882,12 @@ shard.  If this is set, mongos will just skip that shard, instead.
 
 Boolean value, defaults to 0.
 
+Returns this cursor for chaining operations.
+
 =head2 read_preference
 
-    my $cursor = $coll->find()->read_preference($read_preference_object);
-    my $cursor = $coll->find()->read_preference('secondary', [{foo => 'bar'}]);
+    $cursor->read_preference($read_preference_object);
+    $cursor->read_preference('secondary', [{foo => 'bar'}]);
 
 Sets read preference for the cursor's connection.
 
@@ -886,9 +897,9 @@ arguments: the read preference mode and a tag set list, which must be a valid
 mode and tag set list as described in the L<MongoDB::ReadPreference>
 documentation.
 
-Returns $self so that this method can be chained.
+Returns this cursor for chaining operations.
 
-=head2 slave_okay
+=head2 slave_okay (DEPRECATED)
 
     $cursor->slave_okay(1);
 
@@ -896,7 +907,7 @@ If a query can be done on a slave database server.
 
 Boolean value, defaults to 0.
 
-Returns the cursor object
+Returns this cursor for chaining operations.
 
 =head1 QUERY INTROSPECTION AND RESET
 
@@ -909,10 +920,7 @@ stored within the cursor object.
 
 This will tell you the type of cursor used, the number of records the DB had to
 examine as part of this query, the number of records returned by the query, and
-the time in milliseconds the query took to execute.  Requires L<boolean> package.
-
-C<explain> resets the cursor, so calling C<next> or C<has_next> after an explain
-will requery the database.
+the time in milliseconds the query took to execute.
 
 See also core documentation on explain:
 L<http://dochub.mongodb.org/core/explain>.
@@ -1028,7 +1036,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by MongoDB, Inc..
+This software is Copyright (c) 2015 by MongoDB, Inc..
 
 This is free software, licensed under:
 
