@@ -19,7 +19,7 @@ package MongoDB::MongoClient;
 # ABSTRACT: A connection to a MongoDB server or multi-server deployment
 
 use version;
-our $VERSION = 'v0.999.998.1'; # TRIAL
+our $VERSION = 'v0.999.998.2'; # TRIAL
 
 use Moose;
 use MongoDB;
@@ -27,6 +27,7 @@ use MongoDB::Cursor;
 use MongoDB::BSON::Binary;
 use MongoDB::BSON::Regexp;
 use MongoDB::Error;
+use MongoDB::Op::_Command;
 use MongoDB::ReadPreference;
 use MongoDB::WriteConcern;
 use MongoDB::_Topology;
@@ -38,6 +39,7 @@ use Time::HiRes qw/usleep/;
 use Carp 'carp', 'croak';
 use Safe::Isa;
 use Scalar::Util 'reftype';
+use Syntax::Keyword::Junction 'any';
 use boolean;
 use Encode;
 use Try::Tiny;
@@ -52,8 +54,6 @@ use constant {
     NEAREST             => 'nearest',
 };
 
-with 'MongoDB::Role::_Client';
-
 #--------------------------------------------------------------------------#
 # public attributes
 #
@@ -334,18 +334,16 @@ has local_threshold_ms => (
 #pod For core documentation on read preference see
 #pod L<http://docs.mongodb.org/manual/core/read-preference/>.
 #pod
-#pod B<The use of C<read_preference> as a mutator is deprecated.> If given a single
-#pod argument that is a L<MongoDB::ReadPreference> object, the read preference is
-#pod set to that object.  Otherwise, it takes positional arguments: the read
-#pod preference mode and a tag set list, which must be a valid mode and tag set list
-#pod as described in the L<MongoDB::ReadPreference> documentation.
+#pod B<The use of C<read_preference> as a mutator has been removed.>  Read
+#pod preference is read-only.  If you need a different read preference for
+#pod a database or collection, you can specify that in C<get_database> or
+#pod C<get_collection>.
 #pod
 #pod =cut
 
 has read_preference => (
-    is        => 'bare', # public reader implemented manually for back-compatibility
+    is        => 'ro',
     isa       => 'ReadPreference',
-    reader    => '_get_read_preference',
     writer    => '_set_read_preference',
     coerce    => 1,
     lazy      => 1,
@@ -357,28 +355,6 @@ sub _build__read_preference {
     return MongoDB::ReadPreference->new;
 }
 
-sub read_preference {
-    my $self = shift;
-
-    if ( @_ ) {
-        my $type = ref $_[0];
-        if ( $type eq 'MongoDB::ReadPreference' ) {
-            $self->_set_read_preference( $_[0] );
-        }
-        else {
-            my $mode     = shift || 'primary';
-            my $tag_sets = shift;
-            my $rp       = MongoDB::ReadPreference->new(
-                mode => $mode,
-                ( $tag_sets ? ( tag_sets => $tag_sets ) : () )
-            );
-            $self->_set_read_preference($rp);
-        }
-    }
-
-    return $self->_get_read_preference;
-}
-
 # server monitoring
 
 #pod =attr heartbeat_frequency_ms
@@ -683,7 +659,7 @@ sub _build__topology {
         uri                         => $self->_uri,
         type                        => $type,
         server_selection_timeout_ms => $self->server_selection_timeout_ms,
-        latency_threshold_ms        => $self->local_threshold_ms,
+        local_threshold_ms          => $self->local_threshold_ms,
         heartbeat_frequency_ms      => $self->heartbeat_frequency_ms,
         max_wire_version            => $self->_max_wire_version,
         min_wire_version            => $self->_min_wire_version,
@@ -763,8 +739,14 @@ sub BUILD {
 
     $self->_update_write_concern;
 
-    $self->read_preference( $options->{readPreference}, $options->{readPreferenceTags} )
-        if exists $options->{readPreference} || exists $options->{readPreferenceTags};
+    if ( exists $options->{readPreference} ) {
+        my $ts = $options->{readPreferenceTags};
+        my $rp = MongoDB::ReadPreference->new(
+            mode => $options->{readPreference},
+            ( $ts ? ( tag_sets => $ts ) : () ),
+        );
+        $self->_set_read_preference($rp);
+    }
 
     return;
 }
@@ -781,11 +763,6 @@ sub _str_to_bool {
     confess "expected boolean string 'true' or 'false' but instead received '$str'";
 }
 
-sub _use_write_cmd {
-    my ($link) = @_;
-    return $link->min_wire_version <= 2 && 2 <= $link->max_wire_version;
-}
-
 #--------------------------------------------------------------------------#
 # public methods - network communication
 #--------------------------------------------------------------------------#
@@ -827,139 +804,42 @@ sub disconnect {
 #--------------------------------------------------------------------------#
 
 sub send_admin_command {
-    my ($self, $command, $flags, $read_preference) = @_;
+    my ( $self, $command, $read_preference ) = @_;
 
-    $read_preference ||= MongoDB::ReadPreference->new;
-    my $link = $self->_topology->get_readable_link( $read_preference );
-    my $query = MongoDB::_Query->new( spec => $command );
-    $self->_apply_read_prefs( $link, $query, $flags, $read_preference );
-
-    return $self->_try_operation('_send_admin_command', $link, $query->spec, $flags );
-}
-
-sub send_command {
-    my ($self, $db, $command, $flags, $read_preference) = @_;
-
-    $read_preference ||= MongoDB::ReadPreference->new;
-    my $link = $self->_topology->get_readable_link( $read_preference );
-    my $query = MongoDB::_Query->new( spec => $command );
-    $self->_apply_read_prefs( $link, $query, $flags, $read_preference );
-
-    return $self->_try_operation('_send_command', $link, $db, $query->spec, $flags );
-}
-
-sub send_delete {
-    my ( $self, $ns, $op_doc, $write_concern ) = @_;
-    # $op_doc is { q: $query, limit: $limit }
-
-    $write_concern ||= $self->_write_concern;
-    my $link = $self->_topology->get_writable_link;
-
-    return $self->_try_operation('_send_delete', $link, $ns, $op_doc, $write_concern );
-}
-
-sub send_get_more {
-    my ( $self, $address, $ns, $cursor_id, $size ) = @_;
-
-    my $link = $self->_topology->get_specific_link( $address );
-
-    return $self->_try_operation('_send_get_more', $link, $ns, $cursor_id, $size, $self );
-}
-
-sub send_insert {
-    my ( $self, $ns, $docs, $write_concern, $flags, $check_keys ) = @_;
-
-    $docs = [ $docs ] unless ref $docs eq 'ARRAY'; # XXX from BulkWrite
-
-    $write_concern ||= $self->_write_concern;
-    my $link = $self->_topology->get_writable_link;
-
-    return $self->_try_operation('_send_insert', $link, $ns, $docs, $flags, $check_keys, $write_concern );
-}
-
-sub send_kill_cursors {
-    my ( $self, $address, @cursors ) = @_;
-
-    my $link = $self->_topology->get_specific_link( $address );
-
-    return $self->_try_operation('_send_kill_cursors', $link, @cursors );
-}
-
-sub send_update {
-    my ( $self, $ns, $op_doc, $write_concern ) = @_;
-    # $op_doc is { q: $query, u: $update, multi: $multi, upsert: $upsert }
-
-    $write_concern ||= $self->_write_concern;
-    my $link = $self->_topology->get_writable_link;
-
-    return $self->_try_operation('_send_update', $link, $ns, $op_doc, $write_concern );
-}
-
-# XXX eventually, passing $self to _send_query should go away and we should
-# pass in a BSON codec object
-sub send_query {
-    my ($self, $ns, $query, $fields, $skip, $limit, $batch_size, $flags, $read_preference) = @_;
-
-    $read_preference ||= $self->_read_preference || MongoDB::ReadPreference->new;
-    my $link = $self->_topology->get_readable_link( $read_preference );
-
-    $self->_apply_read_prefs( $link, $query, $flags, $read_preference );
-
-    return $self->_try_operation('_send_query', $link, $ns, $query->spec, $fields, $skip, $limit, $batch_size, $flags, $self );
-}
-
-# variants is a hash of wire protocol version to coderef
-sub send_versioned_read {
-    my ( $self, $variants, $read_preference ) = @_;
-
-    $read_preference ||= MongoDB::ReadPreference->new;
-    my $link = $self->_topology->get_readable_link($read_preference);
-
-    # try highest protocol versions first
-    for my $version ( sort { $b <=> $a } keys %$variants ) {
-        if ( $link->accepts_wire_version($version) ) {
-            return $variants->{$version}->($self, $link);
-        }
-    }
-
-    MongoDB::Error->throw(
-        sprintf(
-            "Wire protocol error: server %s selected but doesn't accept protocol(s) %",
-            join( ", ", keys %$variants ),
-            $link->address
-        )
+    my $op   = MongoDB::Op::_Command->new(
+        db_name => 'admin',
+        query => $command,
+        ( $read_preference ? (read_preference => $read_preference) : ()),
     );
+
+    return $self->send_read_op( $op );
 }
 
-sub _apply_read_prefs {
-    my ( $self, $link, $query, $flags, $read_preference ) = @_;
-
-    if ( $link->server->type eq 'Mongos' ) {
-        if ( $read_preference->has_empty_tag_sets ) {
-            if ( $read_preference->mode eq 'primary' ) {
-                $flags->{slave_ok} = 0;
-            }
-            elsif ( $read_preference->mode eq 'secondaryPreferred' ) {
-                $flags->{slave_ok} = 1;
-            }
-            else {
-                $query->set_modifier( '$readPreference' => $read_preference->for_mongos );
-            }
-        }
-        else {
-            $query->set_modifier( '$readPreference' => $read_preference->for_mongos );
-        }
-    }
-    else {
-        $flags->{slave_ok} = 1 if $read_preference->mode ne 'primary';
-    }
+sub send_direct_op {
+    my ( $self, $op, $address ) = @_;
+    my $link = $self->_topology->get_specific_link($address);
+    return $self->_try_op( $op, $link );
 }
 
-sub _try_operation {
-    my ($self, $method, $link, @args) = @_;
+sub send_write_op {
+    my ( $self, $op ) = @_;
+    my $link = $self->_topology->get_writable_link;
+    return $self->_try_op( $op, $link );
+}
+
+sub send_read_op {
+    my ( $self, $op ) = @_;
+    my $link = $self->_topology->get_readable_link($op->read_preference);
+    my $type = $self->_topology->type;
+    return $self->_try_op( $op, $link, $type );
+}
+
+sub _try_op {
+    # $type might be undef; not needed for writes
+    my ($self, $op, $link, $type) = @_;
 
     my $result = try {
-        $self->$method($link, @args);
+        $op->execute( $link, $type );
     }
     catch {
         if ( $_->$_isa("MongoDB::ConnectionError") ) {
@@ -976,269 +856,6 @@ sub _try_operation {
     return $result;
 }
 
-#--------------------------------------------------------------------------#
-# bulk operations
-#--------------------------------------------------------------------------#
-
-# XXX this can be a wrapper to grab link, write concern, and sizes off the
-# link and then move the bulk of this code to _Client
-
-sub send_bulk_queue {
-    my ($self, %args) = @_;
-
-    my $ns = $args{ns};
-    my $queue = $args{queue} || [];
-    my $ordered = $args{ordered};
-    my $write_concern = $args{write_concern} || $self->_write_concern;
-
-    my $link = $self->_topology->get_writable_link;
-
-    my $use_write_cmd = _use_write_cmd($link);
-
-    # If using legacy write ops, then there will never be a valid nModified
-    # result so we set that to undef in the constructor; otherwise, we set it
-    # to 0 so that results accumulate normally. If a mongos on a mixed topology
-    # later fails to set it, results merging will handle it that case.
-    my $result = MongoDB::WriteResult->new( nModified => $use_write_cmd ? 0 : undef, );
-
-    for my $batch ( $ordered ? $self->_batch_ordered($link, $queue) : $self->_batch_unordered($link, $queue) ) {
-        if ($use_write_cmd) {
-            $self->_execute_write_command_batch( $link, $ns, $batch, $result, $ordered, $write_concern );
-        }
-        else {
-            $self->_execute_legacy_batch( $link, $ns, $batch, $result, $ordered, $write_concern );
-        }
-    }
-
-    # only reach here with an error for unordered bulk ops
-    $result->assert_no_write_error;
-
-    # write concern errors are thrown only for the entire batch
-    $result->assert_no_write_concern_error;
-
-    return $result;
-}
-
-my %OP_MAP = (
-    insert => [ insert => 'documents' ],
-    update => [ update => 'updates' ],
-    delete => [ delete => 'deletes' ],
-);
-
-# _execute_write_command_batch may split batches if they are too large and
-# execute them separately
-
-sub _execute_write_command_batch {
-    my ( $self, $link, $ns, $batch, $result, $ordered, $write_concern ) = @_;
-
-    my ( $type, $docs )   = @$batch;
-    my ( $cmd,  $op_key ) = @{ $OP_MAP{$type} };
-
-    my $boolean_ordered = $ordered ? boolean::true : boolean::false;
-    my ($db_name, $coll_name) = $ns =~ m{^([^.]+)\.(.*)$};
-
-    my @left_to_send = ($docs);
-
-    while (@left_to_send) {
-        my $chunk = shift @left_to_send;
-
-        my $cmd_doc = [
-            $cmd    => $coll_name,
-            $op_key => $chunk,
-            ordered => $boolean_ordered,
-            ( $write_concern ? ( writeConcern => $write_concern->as_struct ) : () )
-        ];
-
-        my $cmd_result = try {
-            $self->send_command($db_name, $cmd_doc);
-        }
-        catch {
-            if ( $_->$_isa("MongoDB::_CommandSizeError") ) {
-                if ( @$chunk == 1 ) {
-                    MongoDB::DocumentSizeError->throw(
-                        message  => "document too large",
-                        document => $chunk->[0],
-                    );
-                }
-                else {
-                    unshift @left_to_send, $self->_split_chunk( $link, $chunk, $_->size );
-                }
-            }
-            else {
-                die $_;
-            }
-            return;
-        };
-
-        redo unless $cmd_result; # restart after a chunk split
-
-        my $r = MongoDB::WriteResult->_parse(
-            op       => $type,
-            op_count => scalar @$chunk,
-            result   => $cmd_result,
-        );
-
-        # append corresponding ops to errors
-        if ( $r->count_writeErrors ) {
-            for my $error ( @{ $r->writeErrors } ) {
-                $error->{op} = $chunk->[ $error->{index} ];
-                # convert boolean::true|false back to 1 or 0
-                for my $k (qw/upsert multi/) {
-                    $error->{op}{$k} = 0+ $error->{op}{$k} if exists $error->{op}{$k};
-                }
-            }
-        }
-
-        $result->_merge_result($r);
-        $result->assert_no_write_error if $ordered;
-    }
-
-    return;
-}
-
-sub _split_chunk {
-    my ( $self, $link, $chunk, $size ) = @_;
-
-    my $max_wire_size = $self->MAX_BSON_WIRE_SIZE; # XXX blech
-
-    my $avg_cmd_size       = $size / @$chunk;
-    my $new_cmds_per_chunk = int( $max_wire_size / $avg_cmd_size );
-
-    my @split_chunks;
-    while (@$chunk) {
-        push @split_chunks, [ splice( @$chunk, 0, $new_cmds_per_chunk ) ];
-    }
-
-    return @split_chunks;
-}
-
-sub _batch_ordered {
-    my ($self, $link, $queue) = @_;
-    my @batches;
-    my $last_type = '';
-    my $count     = 0;
-
-    my $max_batch_count = $link->max_write_batch_size;
-
-    for my $op ( @$queue ) {
-        my ( $type, $doc ) = @$op;
-        if ( $type ne $last_type || $count == $max_batch_count ) {
-            push @batches, [ $type => [$doc] ];
-            $last_type = $type;
-            $count     = 1;
-        }
-        else {
-            push @{ $batches[-1][-1] }, $doc;
-            $count++;
-        }
-    }
-
-    return @batches;
-}
-
-sub _batch_unordered {
-    my ($self, $link, $queue) = @_;
-    my %batches = map { ; $_ => [ [] ] } keys %OP_MAP;
-
-    my $max_batch_count = $link->max_write_batch_size;
-
-    for my $op ( @$queue ) {
-        my ( $type, $doc ) = @$op;
-        if ( @{ $batches{$type}[-1] } == $max_batch_count ) {
-            push @{ $batches{$type} }, [$doc];
-        }
-        else {
-            push @{ $batches{$type}[-1] }, $doc;
-        }
-    }
-
-    # insert/update/delete are guaranteed to be in random order on Perl 5.18+
-    my @batches;
-    for my $type ( grep { scalar @{ $batches{$_}[-1] } } keys %batches ) {
-        push @batches, map { [ $type => $_ ] } @{ $batches{$type} };
-    }
-    return @batches;
-}
-
-sub _execute_legacy_batch {
-    my ( $self, $link, $ns, $batch, $result, $ordered, $write_concern ) = @_;
-    my ( $type, $docs ) = @$batch;
-
-    my $method = "send_$type";
-
-    # if write concern is not safe, we have to proxy with a safe one so that
-    # we can interrupt ordered bulks, even while ignoring the actual error
-    my $w_0;
-    if ( ! $write_concern->is_safe ) {
-        my $wc = $write_concern->as_struct;
-        $wc->{w} = 1;
-        $w_0 = MongoDB::WriteConcern->new( $wc );
-    }
-
-    # XXX successive inserts ought to get batched up, up to the max size for batch,
-    # but we have no feedback on max size to know how many to put together. I wonder
-    # if send_insert should return a list of write results, or if it should just
-    # strip out however many docs it can from an arrayref and leave the rest, and
-    # then this code can iterate.
-
-    for my $doc (@$docs) {
-
-        # legacy server doesn't check keys on insert; we fake an error if it happens
-        if ( $type eq 'insert' && ( my $r = $self->_check_no_dollar_keys($doc) ) ) {
-            if ($w_0) {
-                last if $ordered;
-            }
-            else {
-                $result->_merge_result($r);
-                $result->assert_no_write_error if $ordered;
-            }
-            next;
-        }
-
-        my $gle_result = try {
-            $self->$method( $ns, $doc, $w_0 ? $w_0 : $write_concern );
-        }
-        catch {
-            die $_ unless $w_0 && /exceeds maximum size/;
-            undef;
-        };
-
-        # Even for {w:0}, if the batch is ordered we have to break on the first
-        # error, but we don't throw the error to the user.
-        if ( $w_0 ) {
-            last if $ordered && (!$gle_result || $gle_result->count_writeErrors);
-        }
-        else {
-            $result->_merge_result($gle_result);
-            $result->assert_no_write_error if $ordered;
-        }
-    }
-
-    return;
-}
-
-sub _check_no_dollar_keys {
-    my ( $self, $doc ) = @_;
-
-    my @keys = ref $doc eq 'Tie::IxHash' ? $doc->Keys : keys %$doc;
-    if ( my @bad = grep { substr( $_, 0, 1 ) eq '$' } @keys ) {
-        my $errdoc = {
-            index  => 0,
-            errmsg => "Document can't have '\$' prefixed field names: @bad",
-            code   => UNKNOWN_ERROR
-        };
-
-        return MongoDB::WriteResult->new(
-            op_count    => 1,
-            nModified   => undef,
-            writeErrors => [$errdoc]
-        );
-    }
-
-    return;
-}
-
-
 #--------------------------------------------------------------------------#
 # write concern methods
 #--------------------------------------------------------------------------#
@@ -1272,7 +889,7 @@ sub database_names {
     my $max_tries = 3;
     for my $try ( 1 .. $max_tries ) {
         last if try {
-            my $result = $self->get_database('admin')->_try_run_command({ listDatabases => 1 });
+            my $result = $self->send_admin_command([ listDatabases => 1 ])->result;
             if (ref($result) eq 'HASH' && exists $result->{databases}) {
                 @databases = map { $_->{name} } @{ $result->{databases} };
             }
@@ -1287,22 +904,69 @@ sub database_names {
     return @databases;
 }
 
-#pod =method get_database($name)
+#pod =method get_database, db
 #pod
 #pod     my $database = $client->get_database('foo');
+#pod     my $database = $client->get_database('foo', $options);
+#pod     my $database = $client->db('foo', $options);
 #pod
-#pod Returns a L<MongoDB::Database> instance for the database with the given C<$name>.
+#pod Returns a L<MongoDB::Database> instance for the database with the given
+#pod C<$name>.
+#pod
+#pod It takes an optional hash reference of options that are passed to the
+#pod L<MongoDB::Database> constructor.
+#pod
+#pod The C<db> method is an alias for C<get_database>.
 #pod
 #pod =cut
 
 sub get_database {
-    my ($self, $database_name) = @_;
+    my ( $self, $database_name, $options ) = @_;
     return MongoDB::Database->new(
-        _client     => $self,
-        name        => $database_name,
+        read_preference => $self->read_preference,
+        write_concern => $self->_write_concern,
+        ( $options ? %$options : () ),
+        # not allowed to be overridden by options
+        _client       => $self,
+        name          => $database_name,
     );
 }
 
+{ no warnings 'once'; *db = \&get_database }
+
+#pod =method get_namespace, ns
+#pod
+#pod     my $collection = $client->get_namespace('test.foo');
+#pod     my $collection = $client->get_namespace('test.foo', $options);
+#pod     my $collection = $client->ns('test.foo', $options);
+#pod
+#pod Returns a L<MongoDB::Collection> instance for the given namespace.
+#pod The namespace has both the database name and the collection name
+#pod separated with a dot character.
+#pod
+#pod This is a quick way to get a collection object if you don't need
+#pod the database object separately.
+#pod
+#pod It takes an optional hash reference of options that are passed to the
+#pod L<MongoDB::Collection> constructor.  The intermediate L<MongoDB::Database>
+#pod object will be created with default options.
+#pod
+#pod The C<ns> method is an alias for C<get_namespace>.
+#pod
+#pod =cut
+
+sub get_namespace {
+    my ( $self, $ns, $options ) = @_;
+    MongoDB::Error->throw("namespace requires a string argument")
+      unless defined($ns) && length($ns);
+    my ( $db, $coll ) = split /\./, $ns, 2;
+    MongoDB::Error->throw("$ns is not a valid namespace")
+      unless defined($db) && defined($coll);
+    return $self->db($db)->coll( $coll, $options );
+}
+
+{ no warnings 'once'; *ns = \&get_namespace }
+
 #pod =method fsync(\%args)
 #pod
 #pod     $client->fsync();
@@ -1356,7 +1020,7 @@ sub fsync_unlock {
 #pod had been made), sets client attributes as if the username and password had been
 #pod used initially in the client constructor, and reconnects to the configured
 #pod servers.  The authentication mechanism will be MONGO-CR for servers before
-#pod version 2.8 and SCRAM-SHA-1 for 2.8 or later.
+#pod version 3.0 and SCRAM-SHA-1 for 3.0 or later.
 #pod
 #pod Passwords are expected to be cleartext and will be automatically hashed before
 #pod sending over the wire, unless C<$is_digest> is true, which will assume you
@@ -1410,7 +1074,7 @@ MongoDB::MongoClient - A connection to a MongoDB server or multi-server deployme
 
 =head1 VERSION
 
-version v0.999.998.1
+version v0.999.998.2
 
 =head1 SYNOPSIS
 
@@ -1644,11 +1308,10 @@ construct such an object.  The default is mode 'primary'.
 For core documentation on read preference see
 L<http://docs.mongodb.org/manual/core/read-preference/>.
 
-B<The use of C<read_preference> as a mutator is deprecated.> If given a single
-argument that is a L<MongoDB::ReadPreference> object, the read preference is
-set to that object.  Otherwise, it takes positional arguments: the read
-preference mode and a tag set list, which must be a valid mode and tag set list
-as described in the L<MongoDB::ReadPreference> documentation.
+B<The use of C<read_preference> as a mutator has been removed.>  Read
+preference is read-only.  If you need a different read preference for
+a database or collection, you can specify that in C<get_database> or
+C<get_collection>.
 
 =head2 heartbeat_frequency_ms
 
@@ -1783,11 +1446,38 @@ Drops all connections to servers.
 
 Lists all databases on the MongoDB server.
 
-=head2 get_database($name)
+=head2 get_database, db
 
     my $database = $client->get_database('foo');
+    my $database = $client->get_database('foo', $options);
+    my $database = $client->db('foo', $options);
 
-Returns a L<MongoDB::Database> instance for the database with the given C<$name>.
+Returns a L<MongoDB::Database> instance for the database with the given
+C<$name>.
+
+It takes an optional hash reference of options that are passed to the
+L<MongoDB::Database> constructor.
+
+The C<db> method is an alias for C<get_database>.
+
+=head2 get_namespace, ns
+
+    my $collection = $client->get_namespace('test.foo');
+    my $collection = $client->get_namespace('test.foo', $options);
+    my $collection = $client->ns('test.foo', $options);
+
+Returns a L<MongoDB::Collection> instance for the given namespace.
+The namespace has both the database name and the collection name
+separated with a dot character.
+
+This is a quick way to get a collection object if you don't need
+the database object separately.
+
+It takes an optional hash reference of options that are passed to the
+L<MongoDB::Collection> constructor.  The intermediate L<MongoDB::Database>
+object will be created with default options.
+
+The C<ns> method is an alias for C<get_namespace>.
 
 =head2 fsync(\%args)
 
@@ -1822,7 +1512,7 @@ When C<authenticate> is called, it disconnects the client (if any connections
 had been made), sets client attributes as if the username and password had been
 used initially in the client constructor, and reconnects to the configured
 servers.  The authentication mechanism will be MONGO-CR for servers before
-version 2.8 and SCRAM-SHA-1 for 2.8 or later.
+version 3.0 and SCRAM-SHA-1 for 3.0 or later.
 
 Passwords are expected to be cleartext and will be automatically hashed before
 sending over the wire, unless C<$is_digest> is true, which will assume you
@@ -1992,7 +1682,7 @@ MONGODB-CR -- legacy username-password challenge-response
 
 =item *
 
-SCRAM-SHA-1 -- secure username-password challenge-response (2.8+)
+SCRAM-SHA-1 -- secure username-password challenge-response (3.0+)
 
 =item *
 
@@ -2020,7 +1710,7 @@ These mechnisms require a username and password, given either as
 constructor attributes or in the C<host> connection string.
 
 If a username is provided and an authentication mechanism is not specified,
-the client will use SCRAM-SHA-1 for version 2.8 or later servers and will
+the client will use SCRAM-SHA-1 for version 3.0 or later servers and will
 fall back to MONGODB-CR for older servers.
 
     my $mc = MongoDB::MongoClient->new(
@@ -2175,7 +1865,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by MongoDB, Inc..
+This software is Copyright (c) 2015 by MongoDB, Inc..
 
 This is free software, licensed under:
 
