@@ -17,7 +17,7 @@
 package MongoDB::_Server;
 
 use version;
-our $VERSION = 'v0.999.998.1'; # TRIAL
+our $VERSION = 'v0.999.998.2'; # TRIAL
 
 use Moose;
 use MongoDB::_Types;
@@ -33,6 +33,7 @@ use namespace::clean -except => 'meta';
 has address => (
     is       => 'ro',
     isa      => 'HostAddress',
+    coerce   => 1,
     required => 1,
 );
 
@@ -56,7 +57,7 @@ has error => (
 
 has rtt_ms => (
     is      => 'ro',
-    isa     => 'Num',
+    isa     => 'NonNegNum',
     default => 0,
 );
 
