@@ -19,8 +19,9 @@ package MongoDB::_Types;
 # MongoDB Moose type definitions
 
 use version;
-our $VERSION = 'v0.999.998.1'; # TRIAL
+our $VERSION = 'v0.999.998.2'; # TRIAL
 
+use Scalar::Util qw/reftype/;
 use boolean;
 use Moose::Util::TypeConstraints;
 
@@ -46,6 +47,8 @@ enum 'TopologyType',
 
 enum 'ConnectType', [qw/replicaSet direct none/];
 
+enum 'CursorType', [qw/non_tailable tailable tailable_await/];
+
 enum 'ServerType',
   [
     qw/Standalone Mongos PossiblePrimary RSPrimary RSSecondary RSArbiter RSOther RSGhost Unknown/
@@ -60,6 +63,7 @@ class_type 'MongoDBDatabase'   => { class => 'MongoDB::Database' };
 class_type 'booleanpm'         => { class => 'boolean' };
 class_type 'MongoDBQuery'      => { class => 'MongoDB::_Query' };
 class_type 'ReadPreference'    => { class => 'MongoDB::ReadPreference' };
+class_type 'WriteConcern'      => { class => 'MongoDB::WriteConcern' };
 
 subtype ArrayOfHashRef => as 'ArrayRef[HashRef]';
 
@@ -70,8 +74,13 @@ subtype
   where { $_ =~ /^$uri_re$/ },
   message { "Could not parse URI '$_'" };
 
+subtype HashLike => as 'Ref', where { reftype($_) eq 'HASH' };
+
 subtype NonEmptyStr => as 'Str' => where { defined $_ && length $_ };
 
+subtype NonNegNum => as 'Num' => where { defined($_) && $_ >= 0 },
+    message { "value must be a non-negative number" };
+
 # Error string has to be a true value
 subtype ErrorStr => as 'Str' => where { $_ };
 
@@ -100,11 +109,7 @@ coerce booleanpm => from 'Any' => via { boolean($_) };
 coerce IxHash => from 'HashRef'  => via { Tie::IxHash->new(%$_) };
 coerce IxHash => from 'ArrayRef' => via { Tie::IxHash->new(@$_) };
 coerce IxHash => from 'Undef'    => via { Tie::IxHash->new() };
-
-coerce MongoDBQuery => from 'HashRef'  => via { MongoDB::_Query->new( spec => $_ ) };
-coerce MongoDBQuery => from 'ArrayRef' => via { MongoDB::_Query->new( spec => $_ ) };
-coerce MongoDBQuery => from 'IxHash'   => via { MongoDB::_Query->new( spec => $_ ) };
-coerce MongoDBQuery => from 'Undef'    => via { MongoDB::_Query->new( spec => [] ) };
+coerce IxHash => from 'HashLike' => via { Tie::IxHash->new(%$_) };
 
 coerce HostAddressList => from 'ArrayRef' => via {
     [ map { /:/ ? lc $_ : lc "$_:27017" } @$_ ]
@@ -116,13 +121,15 @@ coerce ReadPreference => from 'Str' =>
 coerce ReadPreference => from 'ArrayRef' =>
   via { MongoDB::ReadPreference->new( mode => $_->[0], tag_sets => $_->[1] ) };
 
+coerce WriteConcern => from 'HashRef' => via { MongoDB::WriteConcern->new($_) };
+
 coerce ErrorStr => from 'Str' => via { $_ || "unspecified error" };
 
 no Moose::Util::TypeConstraints;
 
 # Classes for coercions
 require Tie::IxHash;
-require MongoDB::_Query;
 require MongoDB::ReadPreference;
+require MongoDB::WriteConcern;
 
 1;
