@@ -34,17 +34,22 @@ my @testdbs;
 
 # abstract building a connection
 sub build_client {
-    my @args = @_;
-    my $host = exists $ENV{MONGOD} ? $ENV{MONGOD} : 'localhost';
+    my %args = @_;
+    my $host =
+        exists $args{host}  ? delete $args{host}
+      : exists $ENV{MONGOD} ? $ENV{MONGOD}
+      :                       'localhost';
 
     # long query timeout may help spurious failures on heavily loaded CI machines
-    return MongoDB::MongoClient->new(
-        host                        => $host,
-        ssl                         => $ENV{MONGO_SSL},
-        find_master                 => 1,
-        query_timeout               => 60000,
-        server_selection_timeout_ms => 1000,
-        @args,
+    return MongoDB->connect(
+        $host,
+        {
+            ssl                         => $ENV{MONGO_SSL},
+            find_master                 => 1,
+            query_timeout               => 60000,
+            server_selection_timeout_ms => 1000,
+            %args,
+        }
     );
 }
 
@@ -66,7 +71,7 @@ BEGIN {
         $conn->_topology->_dump;
         eval { $conn->_topology->get_writable_link }
             or die "couldn't connect";
-        $conn->get_database("admin")->_try_run_command({ serverStatus => 1 })
+        $conn->get_database("admin")->run_command({ serverStatus => 1 })
             or die "Database has auth enabled\n";
     };
 
@@ -94,7 +99,7 @@ sub server_type {
     my $server_type;
 
     # check database type
-    my $ismaster = $conn->get_database('admin')->_try_run_command({ismaster => 1});
+    my $ismaster = $conn->get_database('admin')->run_command({ismaster => 1});
     if (exists $ismaster->{msg} && $ismaster->{msg} eq 'isdbgrid') {
         $server_type = 'Mongos';
     }
